import { defineStore } from "pinia";
import { ref, computed, reactive, watch } from "vue";
import {
	ArtistTemp,
	YoutubeVideoTemp,
	ReleaseTemp,
	YoutubeChannelTemp
} from "@/types/artist";

const commonPhrases = ["Official MV", "Official Audio", "Official Video"];

const measureStart = name => {
	performance.mark(`${name}-started`);
	console.log(`[MEASURE] START: ${name}`);
};

const measureFinish = name => {
	performance.mark(`${name}-finished`);
	const measure = performance.measure(
		`${name}-duration`,
		`${name}-started`,
		`${name}-finished`
	);
	console.log(`[MEASURE] FINISH: ${name} - ${measure.duration}ms`);
};

type LinkedVideos = { [k: string]: string[] };
type YoutubeVideoMap = { [k: string]: YoutubeVideoTemp };
type HideYoutubeChannel = { [k: string]: boolean };

export const useImportArtistStore = ({ modalUuid }: { modalUuid: string }) =>
	defineStore(`importArtist-${modalUuid}`, () => {
		const artist = ref<ArtistTemp | Partial<ArtistTemp>>({});
		const youtubeChannels = ref<YoutubeChannelTemp[]>([]);
		const hideYoutubeChannel = ref<HideYoutubeChannel>({});
		const youtubeVideoMap = ref<YoutubeVideoMap>({});
		const youtubeVideoIds = ref<string[]>([]);
		const linkedVideos = ref<LinkedVideos>({});
		const recordingsReleasesReleaseGroups = ref<ReleaseTemp[]>([]);
		const filterYoutubeVideos = ref<string>("");
		const recordingFilters = reactive<{
			hideNullLength: boolean;
			hidePartOf: boolean;
		}>({
			hideNullLength: true,
			hidePartOf: true
		});
		const recordingSort = ref<"title" | "length">("title");
		const youtubeVideosSort = ref<"title" | "length">("title");
		const youtubeVideoTitleChanges = reactive<{
			artistDash: boolean;
			parantheses: boolean;
			brackets: boolean;
			commonPhrases: boolean;
		}>({
			artistDash: true,
			parantheses: false,
			brackets: false,
			commonPhrases: true
		});
		const youtubeVideoFilters = reactive<{
			teaser: boolean;
			under45: boolean;
			live: boolean;
			tour: boolean;
		}>({
			teaser: true,
			under45: true,
			live: true,
			tour: true
		});
		const hideYoutubeInfo = ref<boolean>(true);
		const hideMusicbrainzInfo = ref<boolean>(true);
		const youtubeChannelIds = ref<string[]>([]);

		const recordings = computed(() => {
			measureStart("computed recordings");
			const recordingIds = [];
			const recordings = [];
			recordingsReleasesReleaseGroups.value.forEach(release => {
				release.media.forEach(media => {
					media.tracks.forEach(track => {
						const { recording } = track;
						if (!recordingIds.includes(recording.id)) {
							recordingIds.push(recording.id);
							recordings.push(recording);
						}
					});
				});
			});
			measureFinish("computed recordings");
			return recordings;
		});
		const youtubeVideoMapAdjusted = computed(() => {
			measureStart("computed youtubeVideoMapAdjusted");
			const result = Object.fromEntries(
				youtubeVideoIds.value.map(youtubeVideoId => {
					const youtubeVideo = youtubeVideoMap.value[youtubeVideoId];

					let title = youtubeVideo.title.trim();

					if (youtubeVideoTitleChanges.artistDash)
						title = title.replace(
							new RegExp(`${artist.value.name} - `, "i"),
							""
						);
					if (youtubeVideoTitleChanges.parantheses)
						title = title.replaceAll(/\(.+\)/g, "");
					if (youtubeVideoTitleChanges.brackets)
						title = title.replaceAll(/\[.+\]/g, "");
					if (youtubeVideoTitleChanges.commonPhrases) {
						commonPhrases.forEach(commonPhrase => {
							title = title.replace(
								new RegExp(`\\(${commonPhrase}\\)`, "i"),
								""
							);
						});
					}

					title = title.trim();

					let hide = false;
					const lowerTitle = title.toLowerCase();
					if (
						youtubeVideoFilters.teaser &&
						lowerTitle.includes("teaser")
					)
						hide = true;
					if (youtubeVideoFilters.live && lowerTitle.includes("live"))
						hide = true;
					if (youtubeVideoFilters.tour && lowerTitle.includes("tour"))
						hide = true;
					if (
						youtubeVideoFilters.under45 &&
						youtubeVideo.duration < 45
					)
						hide = true;
					// if (hideYoutubeChannel.value[youtubeVideo.rawData.snippet.channelId]) hide = true;

					console.log(youtubeVideoId, "hide", hide);

					return [
						youtubeVideoId,
						{
							...youtubeVideo,
							title,
							hide
						}
					];
				})
			);

			measureFinish("computed youtubeVideoMapAdjusted");
			return result;
		});
		const filteredRecordings = computed(() => {
			measureStart("computed filteredRecordings");
			const result = recordings.value
				.map(recording => {
					let hide = false;
					if (
						recordingFilters.hideNullLength &&
						recording.length === null
					)
						hide = true;
					if (
						recordingFilters.hidePartOf &&
						recording.title.toLowerCase().includes("part of")
					)
						hide = true;
					return {
						...recording,
						hide
					};
				})
				.sort((recordingA, recordingB) => {
					if (recordingSort.value === "title")
						return `${recordingA.title} (${recordingA.disambiguation})`.localeCompare(
							`${recordingB.title} (${recordingB.disambiguation})`
						);
					if (recordingSort.value === "length")
						return recordingA.length - recordingB.length;
					return 0;
				});

			measureFinish("computed filteredRecordings");
			return result;
		});
		const filteredYoutubeVideoIds = computed(() => {
			measureStart("computed filteredYoutubeVideoIds");
			const result = youtubeVideoIds.value
				.filter(youtubeVideoId => {
					const youtubeVideo = youtubeVideoMap.value[youtubeVideoId];
					if (
						hideYoutubeChannel.value[
							youtubeVideo.rawData.snippet.channelId
						]
					)
						return false;
					if (filterYoutubeVideos.value)
						return youtubeVideo.title
							.trim()
							.toLowerCase()
							.includes(filterYoutubeVideos.value.toLowerCase());
					return true;
				})
				.sort((youtubeVideoIdA, youtubeVideoIdB) => {
					const youtubeVideoA =
						youtubeVideoMap.value[youtubeVideoIdA];
					const youtubeVideoB =
						youtubeVideoMap.value[youtubeVideoIdB];
					if (youtubeVideosSort.value === "title")
						return youtubeVideoA.title.localeCompare(
							youtubeVideoB.title
						);
					if (youtubeVideosSort.value === "length")
						return youtubeVideoA.duration - youtubeVideoB.duration;
					return 0;
				})
				.sort((youtubeVideoIdA, youtubeVideoIdB) => {
					const youtubeVideoA =
						youtubeVideoMap.value[youtubeVideoIdA];
					const youtubeVideoB =
						youtubeVideoMap.value[youtubeVideoIdB];
					if (youtubeVideoA.hide && !youtubeVideoB.hide) return 1;
					if (!youtubeVideoA.hide && youtubeVideoB.hide) return -1;
					return 0;
				});
			measureFinish("computed filteredYoutubeVideoIds");
			return result;
		});
		const filteredYoutubeVideosIdsLength = computed(() => {
			measureStart("computed filteredYoutubeVideosIdsLength");
			const result = filteredYoutubeVideoIds.value.filter(
				youtubeVideoId => !youtubeVideoMap.value[youtubeVideoId].hide
			).length;
			measureFinish("computed filteredYoutubeVideosIdsLength");
			return result;
		});

		const setArtist = ({
			name,
			musicbrainzIdentifier,
			youtubeChannels
		}) => {
			artist.value = {
				name,
				musicbrainzIdentifier
			};
			youtubeChannelIds.value = youtubeChannels.map(
				youtubeChannel => youtubeChannel.youtubeChannelId
			);
		};
		const setYoutubeChannels = (_youtubeChannels: YoutubeChannelTemp[]) => {
			youtubeChannels.value = _youtubeChannels;
			youtubeChannels.value.forEach(youtubeChannel => {
				console.log(1221122, youtubeChannel);
				hideYoutubeChannel.value[youtubeChannel.rawData.id] = true;
			});
		};
		const setYoutubeVideos = (youtubeVideos: YoutubeVideoTemp[]) => {
			// youtubeVideos = youtubeVideos.slice(0, 10);
			measureStart("setYoutubeVideos method part one");
			youtubeVideoIds.value = youtubeVideos.map(
				youtubeVideo => youtubeVideo.youtubeId
			);
			measureFinish("setYoutubeVideos method part one");

			measureStart("setYoutubeVideos method part two");
			youtubeVideoMap.value = Object.fromEntries(
				youtubeVideos.map(youtubeVideo => [
					youtubeVideo.youtubeId,
					youtubeVideo
				])
			);
			measureFinish("setYoutubeVideos method part two");
		};
		const setMusicBrainzRecordingsReleasesReleaseGroups = (
			_recordingsReleasesReleaseGroups: ReleaseTemp[]
		) => {
			// _recordingsReleasesReleaseGroups = _recordingsReleasesReleaseGroups.slice(0, 10);
			console.log(
				"albums.getMusicBrainzRecordingsReleasesReleaseGroups",
				_recordingsReleasesReleaseGroups
			);
			recordingsReleasesReleaseGroups.value =
				_recordingsReleasesReleaseGroups;
		};
		const linkVideos = () => {
			filteredRecordings.value.forEach(recording => {
				if (recording.hide) return;
				const lowerRecordingTitle = recording.title
					.toLowerCase()
					.trim();

				filteredYoutubeVideoIds.value.forEach(youtubeVideoId => {
					const youtubeVideo =
						youtubeVideoMapAdjusted.value[youtubeVideoId];
					if (youtubeVideo.hide) return;
					if (
						lowerRecordingTitle ===
						youtubeVideo.title.toLowerCase().trim()
					) {
						if (
							linkedVideos.value[recording.id]?.includes(
								youtubeVideo.youtubeId
							)
						)
							return;
						if (!linkedVideos.value[recording.id])
							linkedVideos.value[recording.id] = [];
						linkedVideos.value[recording.id].push(
							youtubeVideo.youtubeId
						);
					}
				});
			});
			// TODO take into consideration duration
		};

		watch(
			() => recordings.value,
			() => {
				console.log(111222333, "recordings watch");
				recordings.value.forEach(recording => {
					if (!linkedVideos.value[recording.id])
						linkedVideos.value[recording.id] = [];
				});
			}
		);

		return {
			artist,
			youtubeChannels,
			hideYoutubeChannel,
			youtubeVideoMap,
			youtubeVideoIds,
			linkedVideos,
			recordingsReleasesReleaseGroups,
			filterYoutubeVideos,
			recordingFilters,
			recordingSort,
			youtubeVideosSort,
			youtubeVideoTitleChanges,
			youtubeVideoFilters,
			hideYoutubeInfo,
			hideMusicbrainzInfo,
			youtubeChannelIds,
			// getters
			recordings,
			youtubeVideoMapAdjusted,
			filteredRecordings,
			filteredYoutubeVideoIds,
			filteredYoutubeVideosIdsLength,
			// methods
			setArtist,
			setYoutubeChannels,
			setYoutubeVideos,
			setMusicBrainzRecordingsReleasesReleaseGroups,
			linkVideos
		};
	});
